/*

This script parses an Event Trace binary log (.etl file) generated by an IIS tracing session
 (on Windows Server 2003 SP1), groups together all the events associated with the same request, 
 and dumps each single request together with the associated event data.

To start an IIS tracing session, use the 'logman start' shell command, specifying the path to a 
 'provider file' containing the names or GUID's of the desired IIS Tracing Providers.

*/



// Get arguments

var szArgs = WScript.Arguments;
if(szArgs.length!=1)
{
	PrintUsage();
	WScript.Quit(-2);
}

var szTraceFilename=szArgs(0); 



// Execute query

var myLogQuery=new ActiveXObject("MSUtil.LogQuery");
var myETWInput=new ActiveXObject("MSUtil.LogQuery.ETWInputFormat");
myETWInput.fMode="full";

var szQuery="SELECT TO_STRING(Timestamp, 'HH:mm:ss.ll.nn') AS TimestampStr, *, HASHSEQ(ContextID) AS ContextIDSeq FROM " + szTraceFilename + " WHERE ContextId IS NOT NULL ORDER BY ContextIDSeq, Timestamp, EventNumber ASC";

var recordSet=myLogQuery.Execute(szQuery, myETWInput);

//Get the index of the 'ContextId' field

for(var nContextIdIndex=0; nContextIdIndex<recordSet.getColumnCount(); nContextIdIndex++)
 if(recordSet.getColumnName(nContextIdIndex) == "ContextId")
  break;




// Dump requests

var szSepLine="-----------------------------------------------------------------------------";
var cchIndent1=2;
var szIndent1=MakeString(" ", cchIndent1);
var cchIndent2=cchIndent1+2;
var szIndent2=MakeString(" ", cchIndent2);


var nReqs=0;

while(!recordSet.atEnd())
{
	var records=new Array();

	//Get first record
	var record=recordSet.getRecord();
	recordSet.moveNext();

	//Get this ContextId
	var currentContextId=record.getValue("ContextId");

	//Eventually get this Url
	var currentUrl=null;
	if(!record.isNull("RequestURL"))
	 currentUrl=record.getValue("RequestURL");

	//Store all the records with this same ContextId
	records.push(record);
	for(; !recordSet.atEnd(); recordSet.moveNext() )
	{			
		var record=recordSet.getRecord();	

		//Check if this is a new request
		if(record.getValue("ContextId")!=currentContextId)
		 break;

		records.push(record);

		if(currentUrl==null && !record.isNull("RequestURL"))
		 currentUrl=record.getValue("RequestURL");
	}


	//Now dump these values

	//Dump header
	WScript.Echo(szSepLine);
	var timeStamp=new Date(records[0].getValue("Timestamp"));
	nReqs++;
	WScript.Echo("Request n." + nReqs + ": " + ((currentUrl==null)?"":currentUrl) + " " + currentContextId + " " + timeStamp.getYear() + "-" + timeStamp.getMonth() + "-" + timeStamp.getDate() + " " + timeStamp.getHours() + ":" + timeStamp.getMinutes() + ":" + timeStamp.getSeconds() + "\r\n");

	//Dump all the other records
	var lastTimestamp=timeStamp;
	for(var r=0; r<records.length; r++)
	{
		var record=records[r];

		//Dump EventName and EventTypeName
		WScript.Echo( szIndent1 + record.getValue("EventName") + ": " + record.getValue("EventTypeName") + " - " + record.getValue("EventTypeDescription"));

		//Dump individual fields
		for(var i=nContextIdIndex+1; i<recordSet.getColumnCount(); i++)
		 if(!record.isNull(i))
		 {
			WScript.Echo( szIndent2 + recordSet.getColumnName(i) + ": " + record.getValue(i) );
		 }

		//Dump Timestamp
		WScript.Echo( szIndent2 + "Timestamp: " + record.getValue("TimestampStr"));

		WScript.Echo("");

		lastTimeStamp=new Date(record.getValue("Timestamp"));
	}

	WScript.Echo(" Total time: " + (lastTimeStamp.valueOf()-timeStamp.valueOf()) + " msecs");

	//Prompt the user to press a key
	if(!recordSet.atEnd())
	{
		WScript.Echo(szSepLine);
		WScript.Echo("[Hit ENTER...]");
		WScript.StdIn.ReadLine();
	}

	WScript.Echo("");	
}

recordSet.close();

WScript.Echo(szSepLine);

//END

function MakeString(szString, cLen)
{
	var szRet="";
	for(var i=0; i<cLen; i++)
	 szRet+=szString;

	return szRet;
}

function PrintUsage()
{
	WScript.Echo("Usage: DumpTraceReqs <trace_filename>");
	WScript.Echo("\r\nExample: DumpTraceReqs myTrace.etl\r\n\r\n");
}